/* Dacus
 * Modified by ElectroDacus (Dacian Todea) for the SBMS and DMPPT specific hardware
  ******************************************************************************
  * @file    stm320518_eval_i2c_ee.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    20-April-2012
  * @brief   This file provides a set of functions needed to manage an I2C M24LR64 
  *          EEPROM memory.
  *          This is a modified version for the ElectroDacus SBMS4080 project.
  *          
  *          ===================================================================
  *          Notes: 
  *           - This driver is intended for STM32F0xx families devices only.
  *           - The I2C EEPROM memory (M24LR64) is available on RF EEPROM daughter 
  *             board (ANT7-M24LR-A) provided with the EVAL board, to use this 
  *             driver you have to connect the ANT7-M24LR-A to CN12 connector.   
  *          ===================================================================
  *              
  *          It implements a high level communication layer for read and write 
  *          from/to this memory. The needed STM32F0xx hardware resources (I2C and 
  *          GPIO) are defined in stm320518_eval.h file, and the initialization is 
  *          performed in sDMPPT_LowLevel_Init() function declared in stm320518_eval.c
  *          file.
  *          You can easily tailor this driver to any other development board, 
  *          by just adapting the defines for hardware resources and 
  *          sDMPPT_LowLevel_Init() function.
  *        
  *          @note In this driver, basic read and write functions (sDMPPT_ReadBuffer()
  *                and sDMPPT_WritePage()) use Polling mode to perform the data transfer
  *                to/from EEPROM memory.
  *            
  *     +-----------------------------------------------------------------+
  *     |                        Pin assignment                           |
  *     +---------------------------------------+-----------+-------------+
  *     |  STM32F0xx I2C Pins                   |   sDMPPT     |   Pin       |
  *     +---------------------------------------+-----------+-------------+
  *     | .                                     |   E0(GND) |    1  (0V)  |
  *     | .                                     |   AC0     |    2        |
  *     | .                                     |   AC1     |    3        |
  *     | .                                     |   VSS     |    4  (0V)  |
  *     | sDMPPT_I2C_SDA_PIN/ SDA                  |   SDA     |    5        |
  *     | sDMPPT_I2C_SCL_PIN/ SCL                  |   SCL     |    6        |
  *     | .                                     |   E1(GND) |    7  (0V)  |
  *     | .                                     |   VDD     |    8 (3.3V) |
  *     +---------------------------------------+-----------+-------------+  
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * sDMPPT the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "DMPPT_i2c2.h"


__IO uint16_t  sDMPPTAddress = 0;
__IO uint32_t  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
__IO uint16_t  sDMPPTDataNum;



void sDMPPT_LowLevel_DeInit(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* sDMPPT_I2C Peripheral Disable */
  I2C_Cmd(sDMPPT_I2C, DISABLE);

  /* sDMPPT_I2C DeInit */
  I2C_DeInit(sDMPPT_I2C);

  //RCC_APB1PeriphResetCmd(sDMPPT_I2C_CLK, ENABLE);
  /*!< sDMPPT_I2C Periph clock disable */
  RCC_APB1PeriphClockCmd(sDMPPT_I2C_CLK, DISABLE);


  /*!< GPIO configuration */
  /*!< Configure sDMPPT_I2C pins: SCL */
  GPIO_InitStructure.GPIO_Pin = sDMPPT_I2C_SCL_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(sDMPPT_I2C_SCL_GPIO_PORT, &GPIO_InitStructure);

  /*!< Configure sDMPPT_I2C pins: SDA */
  GPIO_InitStructure.GPIO_Pin = sDMPPT_I2C_SDA_PIN;
  GPIO_Init(sDMPPT_I2C_SDA_GPIO_PORT, &GPIO_InitStructure);
}

/**
* @brief Initializes peripherals used by the I2C EEPROM driver.
* @param None
* @retval None
*/
void sDMPPT_LowLevel_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Configure the I2C clock source. The clock is derived from the HSI */
  RCC_I2CCLKConfig(RCC_I2C1CLK_HSI);

  /*!< sDMPPT_I2C_SCL_GPIO_CLK and sDMPPT_I2C_SDA_GPIO_CLK Periph clock enable */
  RCC_AHBPeriphClockCmd(sDMPPT_I2C_SCL_GPIO_CLK | sDMPPT_I2C_SDA_GPIO_CLK, ENABLE);

  /*!< sDMPPT_I2C Periph clock enable */
  RCC_APB1PeriphClockCmd(sDMPPT_I2C_CLK, ENABLE);






  /*!< GPIO configuration */
  /*!< Configure sDMPPT_I2C pins: SCL */
  GPIO_InitStructure.GPIO_Pin = sDMPPT_I2C_SCL_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
  //dacus next line
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL; //was UP on SBMS4080
  GPIO_Init(sDMPPT_I2C_SCL_GPIO_PORT, &GPIO_InitStructure);

  /*!< Configure sDMPPT_I2C pins: SDA */
  GPIO_InitStructure.GPIO_Pin = sDMPPT_I2C_SDA_PIN;
  GPIO_Init(sDMPPT_I2C_SDA_GPIO_PORT, &GPIO_InitStructure);

  /* Connect PXx to I2C_SCL*/
  GPIO_PinAFConfig(sDMPPT_I2C_SCL_GPIO_PORT, sDMPPT_I2C_SCL_SOURCE, sDMPPT_I2C_SCL_AF);

  /* Connect PXx to I2C_SDA*/
  GPIO_PinAFConfig(sDMPPT_I2C_SDA_GPIO_PORT, sDMPPT_I2C_SDA_SOURCE, sDMPPT_I2C_SDA_AF);

}




/** @defgroup STM320518_EVAL_I2C_EE_Private_Functions
  * @{
  */ 

/**
  * @brief  DeInitializes peripherals used by the I2C EEPROM driver.
  * @param  None
  * @retval None
  */
void sDMPPT_DeInit(void)
{
  sDMPPT_LowLevel_DeInit();
}

/**
  * @brief  Initializes peripherals used by the I2C EEPROM driver.
  * @param  None
  * @retval None
  */
void sDMPPT_Init(void)
{ 
  I2C_InitTypeDef  I2C_InitStructure;
  sDMPPT_DeInit(); //dacus
  
  sDMPPT_LowLevel_Init();
  
  I2C_Cmd(I2C2, ENABLE);
  I2C_Cmd(I2C2, DISABLE);
  /*!< I2C configuration */
  /* sDMPPT_I2C configuration */
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_AnalogFilter = I2C_AnalogFilter_Enable;
  I2C_InitStructure.I2C_DigitalFilter = 0x00;
  I2C_InitStructure.I2C_OwnAddress1 = 0xA0;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_Timing = sDMPPT_I2C_TIMING;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  

  /* Apply sDMPPT_I2C configuration after enabling it */
  I2C_Init(sDMPPT_I2C, &I2C_InitStructure);

  //I2C_StretchClockCmd(I2C2, ENABLE); //dacus

  /* sDMPPT_I2C Peripheral Enable */
  I2C_Cmd(sDMPPT_I2C, ENABLE);
  
  NVIC_InitTypeDef NVIC_InitStructure;
  NVIC_InitStructure.NVIC_IRQChannel = I2C2_EV_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);



  /*!< Select the EEPROM address */
  //sDMPPTAddress = sDMPPT_HW_ADDRESS;
}

/**
  * @brief  Reads a block of data from the EEPROM.
  * @param  pBuffer: pointer to the buffer that receives the data read from 
  *         the EEPROM.
  * @param  ReadAddr: EEPROM's internal address to start reading from.
  * @param  NumByteToRead: pointer to the variable holding number of bytes to 
  *         be read from the EEPROM.
  *
  * @retval sDMPPT_OK (0) if operation is correctly performed, else return value
  *         different from sDMPPT_OK (0) or the timeout user callback.
  */
uint32_t sDMPPT_ReadBuffer(uint8_t* pBuffer, uint16_t ReadAddr, uint16_t* NumByteToRead)
{  
  uint32_t NumbOfSingle = 0, Count = 0, DataNum = 0, StartCom = 0;
  
  /* Get number of reload cycles */
  Count = (*NumByteToRead) / 255;  
  NumbOfSingle = (*NumByteToRead) % 255;
  
#ifdef sDMPPT_M24C08
  
  /* Configure slave address, nbytes, reload and generate start */
  I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 1, I2C_SoftEnd_Mode, I2C_Generate_Start_Write);
  
  /* Wait until TXIS flag is set */
  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TXIS) == RESET)
  {
    if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
  }
  
  /* Send memory address */
  I2C_SendData(sDMPPT_I2C, (uint8_t)ReadAddr);
  
#elif defined(sDMPPT_M24M01) || defined(sDMPPT_M24C64_32) || defined (sDMPPT_1)

  /* Configure slave address, nbytes, reload and generate start */
  I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 1, I2C_SoftEnd_Mode, I2C_Generate_Start_Write);

  /* Wait until TXIS flag is set */
  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TXIS) == RESET)
  {
    if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
  }

  /* Send MSB of memory address */
  I2C_SendData(sDMPPT_I2C, (uint8_t)(ReadAddr));
  //----------------------------------------------
//delay_ms(1);



#endif /*!< sDMPPT_M24C08 */
  
  /* Wait until TC flag is set */
  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TC) == RESET)
  {
    if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
  }  

  /* If number of Reload cycles is not equal to 0 */
  if (Count != 0)
  {
    /* Starting communication */
    StartCom = 1;

    /* Wait until all reload cycles are performed */
    while( Count != 0)
    { 
      /* If a read transfer is performed */
      if (StartCom == 0)      
      {
        /* Wait until TCR flag is set */
        sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
        while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TCR) == RESET)
        {
          if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
        }
      }      

      /* if remains one read cycle */
      if ((Count == 1) && (NumbOfSingle == 0))
      {
        /* if starting communication */
        if (StartCom != 0)
        {
          /* Configure slave address, end mode and start condition */
          I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 255, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);
        }
        else
        {
          /* Configure slave address, end mode */
          I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 255, I2C_AutoEnd_Mode, I2C_No_StartStop);
        }
      }
      else 
      {
        /* if starting communication */
        if (StartCom != 0)
        {
          /* Configure slave address, end mode and start condition */
          I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 255, I2C_Reload_Mode, I2C_Generate_Start_Read);
        }
        else
        {
          /* Configure slave address, end mode */
          I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 255, I2C_Reload_Mode, I2C_No_StartStop);
        } 
      }
      
      /* Update local variable */
      StartCom = 0;      
      DataNum = 0;

      /* Wait until all data are received */
      while (DataNum != 255)
      {        
        /* Wait until RXNE flag is set */
        sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
        while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_RXNE) == RESET)
        {
          if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
        }
        
        /* Read data from RXDR */
        pBuffer[DataNum]= I2C_ReceiveData(sDMPPT_I2C);
        
        /* Update number of received data */
        DataNum++;
        (*NumByteToRead)--;
      }      
      /* Update Pointer of received buffer */ 
      pBuffer += DataNum;  
      
      /* update number of reload cycle */
      Count--;
    }
    
    /* If number of single data is not equal to 0 */
    if (NumbOfSingle != 0)
    {            
      /* Wait until TCR flag is set */
      sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
      while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TCR) == RESET)
      {
        if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
      }
      
      /* Update CR2 : set Nbytes and end mode */
      I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, (uint8_t)(NumbOfSingle), I2C_AutoEnd_Mode, I2C_No_StartStop);
      
      /* Reset local variable */
      DataNum = 0;
      
      /* Wait until all data are received */
      while (DataNum != NumbOfSingle)
      {        
        /* Wait until RXNE flag is set */
        sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
        while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_RXNE) == RESET)
        {
          if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
        }
        
        /* Read data from RXDR */
        pBuffer[DataNum]= I2C_ReceiveData(sDMPPT_I2C);
        
        /* Update number of received data */
        DataNum++;
        (*NumByteToRead)--;
      } 
    }
  }
  else
  {
    /* Update CR2 : set Slave Address , set read request, generate Start and set end mode */
    I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, (uint32_t)(NumbOfSingle), I2C_AutoEnd_Mode, I2C_Generate_Start_Read);
    
    /* Reset local variable */
    DataNum = 0;
    
    /* Wait until all data are received */
    while (DataNum != NumbOfSingle)
    {
      /* Wait until RXNE flag is set */
      sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
      while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_RXNE) == RESET)
      {
        if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
      }
      
      /* Read data from RXDR */
      pBuffer[DataNum]= I2C_ReceiveData(sDMPPT_I2C);
      
      /* Update number of received data */
      DataNum++;
      (*NumByteToRead)--;
    }    
  }  
  
  /* Wait until STOPF flag is set */
  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_STOPF) == RESET)
  {
    if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
  }
  
  /* Clear STOPF flag */
  I2C_ClearFlag(sDMPPT_I2C, I2C_ICR_STOPCF);
  
  /* If all operations OK, return sDMPPT_OK (0) */
  return sDMPPT_OK;
}

/**
  * @brief  Writes more than one byte to the EEPROM with a single WRITE cycle.
  *
  * @note   The number of bytes (combined to write start address) must not 
  *         cross the EEPROM page boundary. This function can only write into
  *         the boundaries of an EEPROM page.
  * @note   This function doesn't check on boundaries condition (in this driver 
  *         the function sDMPPT_WriteBuffer() which calls sDMPPT_WritePage() is
  *         responsible of checking on Page boundaries).
  * 
  * @param  pBuffer: pointer to the buffer containing the data to be written to 
  *         the EEPROM.
  * @param  WriteAddr: EEPROM's internal address to write to.
  * @param  NumByteToWrite: pointer to the variable holding number of bytes to 
  *         be written into the EEPROM.
  *
  * @retval sDMPPT_OK (0) if operation is correctly performed, else return value
  *         different from sDMPPT_OK (0) or the timeout user callback.
  */
uint32_t sDMPPT_WritePage(uint8_t* pBuffer, uint16_t WriteAddr, uint8_t* NumByteToWrite)
{   
  uint32_t DataNum = 0;
  
#ifdef sDMPPT_M24C08
  
  /* Configure slave address, nbytes, reload and generate start */
  I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 1, I2C_Reload_Mode, I2C_Generate_Start_Write);
  
  /* Wait until TXIS flag is set */
  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TXIS) == RESET)
  {
    if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
  }
  
  /* Send memory address */
  I2C_SendData(sDMPPT_I2C, (uint8_t)WriteAddr);
  
#elif defined(sDMPPT_M24M01) || defined(sDMPPT_M24C64_32) || defined (sDMPPT_1)
  
  /* Configure slave address, nbytes, reload and generate start reload_mode*/
  I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 1, I2C_Reload_Mode, I2C_Generate_Start_Write);

  /* Wait until TXIS flag is set */
  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TXIS) == RESET)
  {
    if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
  }
  
  /* Send MSB of memory address */
 // I2C_SendData(sDMPPT_I2C, (uint8_t)((WriteAddr & 0xFF00) >> 8));
  
  /* Wait until TXIS flag is set */
 // sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
 // while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TXIS) == RESET)
 // {
 //   if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
 // }
  
  /* Send LSB of memory address  */
  I2C_SendData(sDMPPT_I2C, (uint8_t)WriteAddr );
  delay_ms(1); //dacus delay
  
#endif /*!< sDMPPT_M24C08 */
  
  /* Wait until TCR flag is set */
  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TCR) == RESET)
  {
    if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
  }

  /* Update CR2 : set Slave Address , set write request, generate Start and set end mode */
  I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, (uint8_t)(*NumByteToWrite), I2C_AutoEnd_Mode, I2C_No_StartStop);
  
  while (DataNum != (*NumByteToWrite))
  {      
    /* Wait until TXIS flag is set */
    sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
    while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_TXIS) == RESET)
    {
      if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
    }  
    
    /* Write data to TXDR */
    I2C_SendData(sDMPPT_I2C, (uint8_t)(pBuffer[DataNum]));
    
    /* Update number of transmitted data */
    DataNum++;   
  }  
  
  /* Wait until STOPF flag is set */
  sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_STOPF) == RESET)
  {
    if((sDMPPTTimeout--) == 0) return sDMPPT_TIMEOUT_UserCallback();
  }   
  
  /* Clear STOPF flag */
  I2C_ClearFlag(sDMPPT_I2C, I2C_ICR_STOPCF);
  
  /* If all operations OK, return sDMPPT_OK (0) */
  return sDMPPT_OK;
}

/**
  * @brief  Writes buffer of data to the I2C EEPROM.
  * @param  pBuffer: pointer to the buffer  containing the data to be written 
  *         to the EEPROM.
  * @param  WriteAddr: EEPROM's internal address to write to.
  * @param  NumByteToWrite: number of bytes to write to the EEPROM.
  * @retval None
  */
void sDMPPT_WriteBuffer(uint8_t* pBuffer, uint16_t WriteAddr, uint16_t NumByteToWrite)
{
  uint16_t NumOfPage = 0, NumOfSingle = 0, count = 0;
  uint16_t Addr = 0;
  
  Addr = WriteAddr % sDMPPT_PAGESIZE;
  count = sDMPPT_PAGESIZE - Addr;
  NumOfPage =  NumByteToWrite / sDMPPT_PAGESIZE;
  NumOfSingle = NumByteToWrite % sDMPPT_PAGESIZE;
  
  /*!< If WriteAddr is sDMPPT_PAGESIZE aligned  */
  if(Addr == 0) 
  {
    /*!< If NumByteToWrite < sDMPPT_PAGESIZE */
    if(NumOfPage == 0) 
    {
      /* Store the number of data to be written */
      sDMPPTDataNum = NumOfSingle;
      /* Start writing data */
      sDMPPT_WritePage(pBuffer, WriteAddr, (uint8_t*)(&sDMPPTDataNum));
      sDMPPT_WaitEepromStandbyState();
    }
    /*!< If NumByteToWrite > sDMPPT_PAGESIZE */
    else  
    {
      while(NumOfPage--)
      {
        /* Store the number of data to be written */
        sDMPPTDataNum = sDMPPT_PAGESIZE;
        sDMPPT_WritePage(pBuffer, WriteAddr, (uint8_t*)(&sDMPPTDataNum));
        sDMPPT_WaitEepromStandbyState();
        WriteAddr +=  sDMPPT_PAGESIZE;
        pBuffer += sDMPPT_PAGESIZE;
      }
      
      if(NumOfSingle!=0)
      {
        /* Store the number of data to be written */
        sDMPPTDataNum = NumOfSingle;
        sDMPPT_WritePage(pBuffer, WriteAddr, (uint8_t*)(&sDMPPTDataNum));
        sDMPPT_WaitEepromStandbyState();
      }
    }
  }
  /*!< If WriteAddr is not sDMPPT_PAGESIZE aligned  */
  else 
  {
    /*!< If NumByteToWrite < sDMPPT_PAGESIZE */
    if(NumOfPage== 0) 
    {
      /*!< If the number of data to be written is more than the remaining space 
      in the current page: */
      if (NumByteToWrite > count)
      {
        /* Store the number of data to be written */
        sDMPPTDataNum = count;
        /*!< Write the data conained in same page */
        sDMPPT_WritePage(pBuffer, WriteAddr, (uint8_t*)(&sDMPPTDataNum));
        sDMPPT_WaitEepromStandbyState();
        
        /* Store the number of data to be written */
        sDMPPTDataNum = (NumByteToWrite - count);
        /*!< Write the remaining data in the following page */
        sDMPPT_WritePage((uint8_t*)(pBuffer + count), (WriteAddr + count), (uint8_t*)(&sDMPPTDataNum));
        sDMPPT_WaitEepromStandbyState();
      }      
      else      
      {
        /* Store the number of data to be written */
        sDMPPTDataNum = NumOfSingle;
        sDMPPT_WritePage(pBuffer, WriteAddr, (uint8_t*)(&sDMPPTDataNum));
        sDMPPT_WaitEepromStandbyState();
      }     
    }
    /*!< If NumByteToWrite > sDMPPT_PAGESIZE */
    else
    {
      NumByteToWrite -= count;
      NumOfPage =  NumByteToWrite / sDMPPT_PAGESIZE;
      NumOfSingle = NumByteToWrite % sDMPPT_PAGESIZE;
      
      if(count != 0)
      {  
        /* Store the number of data to be written */
        sDMPPTDataNum = count;
        sDMPPT_WritePage(pBuffer, WriteAddr, (uint8_t*)(&sDMPPTDataNum));
        sDMPPT_WaitEepromStandbyState();
        WriteAddr += count;
        pBuffer += count;
      } 
      
      while(NumOfPage--)
      {
        /* Store the number of data to be written */
        sDMPPTDataNum = sDMPPT_PAGESIZE;
        sDMPPT_WritePage(pBuffer, WriteAddr, (uint8_t*)(&sDMPPTDataNum));
        sDMPPTTimeout = sDMPPT_LONG_TIMEOUT;
        sDMPPT_WaitEepromStandbyState();
        WriteAddr +=  sDMPPT_PAGESIZE;
        pBuffer += sDMPPT_PAGESIZE;
      }
      if(NumOfSingle != 0)
      {
        /* Store the number of data to be written */
        sDMPPTDataNum = NumOfSingle;
        sDMPPT_WritePage(pBuffer, WriteAddr, (uint8_t*)(&sDMPPTDataNum));
        sDMPPT_WaitEepromStandbyState();
      }
    }
  }  
}

/**
  * @brief  Wait for EEPROM Standby state.
  * 
  * @note  This function allows to wait and check that EEPROM has finished the 
  *        last operation. It is mostly used after Write operation: after receiving
  *        the buffer to be written, the EEPROM may need additional time to actually
  *        perform the write operation. During this time, it doesn't answer to
  *        I2C packets addressed to it. Once the write operation is complete
  *        the EEPROM responds to its address.
  * 
  * @param  None
  *
  * @retval sDMPPT_OK (0) if operation is correctly performed, else return value
  *         different from sDMPPT_OK (0) or the timeout user callback.
  */
uint32_t sDMPPT_WaitEepromStandbyState(void)
{
  __IO uint32_t sDMPPTTrials = 0;
  
  /* Keep looping till the slave acknowledge his address or maximum number 
  of trials is reached (this number is defined by sDMPPT_MAX_TRIALS_NUMBER define
  in stm32373c_eval_i2c_ee.h file) */
  
  /* Configure CR2 register : set Slave Address and end mode */
  I2C_TransferHandling(sDMPPT_I2C, sDMPPTAddress, 0, I2C_AutoEnd_Mode, I2C_No_StartStop);
  
  do
  { 
    /* Initialize sDMPPTTimeout */
    sDMPPTTimeout = sDMPPT_FLAG_TIMEOUT;
    
    /* Clear NACKF */
    I2C_ClearFlag(sDMPPT_I2C, I2C_ICR_NACKCF | I2C_ICR_STOPCF);
    
    /* Generate start */
    I2C_GenerateSTART(sDMPPT_I2C, ENABLE);
    
    /* Wait until timeout elapsed */
    while (sDMPPTTimeout-- != 0);
    
    /* Check if the maximum allowed numbe of trials has bee reached */
    if (sDMPPTTrials++ == sDMPPT_MAX_TRIALS_NUMBER)
    {
      /* If the maximum number of trials has been reached, exit the function */
      return sDMPPT_TIMEOUT_UserCallback();
    }
  }
  while(I2C_GetFlagStatus(sDMPPT_I2C, I2C_ISR_NACKF) != RESET);
  
  /* Clear STOPF */
  I2C_ClearFlag(sDMPPT_I2C, I2C_ICR_STOPCF);
  
  /* Return sDMPPT_OK if device is ready */
  return sDMPPT_OK;
}

//#ifdef USE_DEFAULT_TIMEOUT_CALLBACK
/**
  * @brief  Basic management of the timeout situation.
  * @param  None.
  * @retval None.
  */
/* dacus is in main for now

uint32_t sDMPPT_TIMEOUT_UserCallback(void)
{
   //Block communication and all processes
  while (1)
  {
  }
}*/
//#endif // USE_DEFAULT_TIMEOUT_CALLBACK



/**
  * @}
  */
  
/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */  

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
